<?php
// $Id: select.inc,v 1.18.2.19.2.16 2008/06/27 03:19:11 quicksketch Exp $
/**
 * edited by madjoe on 2008/07/16 04:12:10.
 * Create a set of form items to be displayed on the form for editing this
 * component. Use care naming the form items, as this correlates directly to the
 * database schema. The component "Name" and "Description" fields are added to
 * every component type and are not necessary to specify here (although they may
 * be overridden if desired).
 * @return
 *   An array of form items to be displayed on the edit component page.
 */
function _webform_edit_select($currfield) {
  global $user;
  $edit_fields = array();
  $edit_fields['extra']['items'] = array(
    '#type' => 'textarea',
    '#title' => t("Options"),
    '#default_value' => $currfield['extra']['items'],
    '#description' => t('A list of selectable options. One option per line. Key-value pairs may be entered seperated by pipes. i.e. safe_key|Some readable option') .'<br />'. webform_help('webform/helptext#variables'),
    '#cols' => 60,
    '#rows' => 5,
    '#weight' => -2,
    '#required' => FALSE,
  );

  if ($user->uid == 1 OR user_access('use PHP for additional processing')){
    $edit_fields['extra']['php'] = array(
      '#type' => 'fieldset',
      '#title' => 'PHP Options',
      '#collapsible' => 'TRUE',
      '#collapsed' => 'TRUE',
      '#weight' => -1,   
      '#description' => t("These entries allow a php function to generate the selection values and the default option.  Powerful, yes. Dangerous if you don't know what you are doing or you let UNTRUSTED people create these forms. If that seems like a bad idea do not allow 'use PHP for additional processing' in the access controls. <strong>The values generated by this PHP overrides values in the Default value and Options boxes above</strong>") .'<br />',
      );
    
    $edit_fields['extra']['php']['phpoptions'] = array(
      '#type' => 'textarea',
      '#title' => t("Options from PHP"),
      '#default_value' => $currfield['extra']['php']['phpoptions'],
      '#description' => t('A php expression that generates an array of options. If the array has strings for keys the keys will be used as the value returned from the select element.  These options will replace any other options added in the above box.  Do not use the <?php  ?> tags an example would be "return array_merge($somearray, $someotherarray );" or "return some_function_that_returns_array($arg);" The PHP is evaluated before the variable subsitution occurs so "return array("user1", "user2", "%username")" will add the users username to a selection including user1 and user2. These strings are evaluated with the php eval statement so the variables and functions that are available when the form is filled out can be used in the php string') .'<br />',
      '#cols' => 60,
      '#rows' => 5,
      '#weight' => -1,
      '#required' => FALSE,
    );
  
    $edit_fields['extra']['php']['phpdefault'] = array(
      '#type' => 'textarea',
      '#title' => t("Default Option from PHP"),
      '#default_value' => $currfield['extra']['php']['phpdefault'],
      '#description' => t('A php expression that generates an array of options. In general there will only be one option in the array but there can be multiple defaults if multiple selections are allowed.  If the array has strings for keys the keys will be used as the value returned from the select element.  These options will replace any other options added in the above box.  Do not use the <?php  ?> tags an example would be "return array_merge($somearray, $someotherarray );" or "return some_function_that_returns_array($arg);" These strings are evaluated with the php eval statement so the variables and functions that are available when the form is filled out can be used in the php string') .'<br />',
      '#cols' => 60,
      '#rows' => 5,
      '#weight' => -2,
      '#required' => FALSE,
    );
  }
  $edit_fields['value'] = array(
    '#type' => 'textfield',
    '#title' => t("Default value"),
    '#default_value' => $currfield['value'],
    '#description' => t('The default value of the field. For multiple selects use commas to separate multiple defaults.') .'<br />'. webform_help('webform/helptext#variables'),
    '#size' => 60,
    '#maxlength' => 256,
    '#weight' => 0,
  );
  $edit_fields['extra']['multiple'] = array(
    '#type' => 'checkbox',
    '#title' => t("Multiple"),
    '#return_value' => 'Y',
    '#default_value' => ($currfield['extra']['multiple'] == 'Y' ? TRUE : FALSE),
    '#description' => t('Check this option if the user should be allowed to choose multiple values.'),
  );
  $edit_fields['extra']['aslist'] = array(
    '#type' => 'checkbox',
    '#title' => t("Listbox"),
    '#return_value' => 'Y',
    '#default_value' => ($currfield['extra']['aslist'] == 'Y' ? TRUE : FALSE),
    '#description' => t('Check this option if you want the select component to be of listbox type instead of radiobuttons or checkboxes.'),
  );
  $edit_fields['extra']['email'] = array(
    '#type' => 'checkbox',
    '#title' => t("E-mail a submission copy"),
    '#return_value' => 1,
    '#default_value' => $currfield['extra']['email'],
    '#description' => t('Check this option if this component contains an e-mail address that should get a copy of the submission. Emails are sent individually so other emails will not be shown to the recipient.') ." ".
                      t('To use the option with a select component, you must use key-value pairs seperated by pipes. i.e. user@example.com|Sample user.'),
  );
  return $edit_fields;
}

function _webform_edit_validate_select($form_values) {
  global $user;
  
  // Currently no validation for selects.
  // TODO: Validate e-mail addresses when used as keys?
  // this is made more complex since some people may not see the php options if they do not have access
   if (($user->uid == 1) OR user_access('use PHP for additional processing')){ 
       if (!(strlen($form_values['extra']['items']) OR strlen($form_values['extra']['php']['phpoptions']))){
        form_set_error('field][extra][items', t('There are no valid options. Specify values either in Options or Options from PHP (Under PHP Options)'));
        return false;  
       } 
      }else {
        if (!(strlen($form_values['extra']['items']) )){
         form_set_error('field][extra][items', t('There are no valid options. Specify at least one value in Options'));
         return false;      
      }
    }
   $rows = explode("\n", _webform_filter_values($form_values['extra']['items'], NULL, NULL, FALSE));
   foreach ($rows as $row) {
     $row = trim($row);
     if (preg_match('/^(.+)?\|(.*)$/', $row, $matches)) {
       if (preg_match('/ |"/', $matches[1])) {
         form_set_error('field][extra][items', t('The options for this select contain illegal characters (quotes or spaces). Specify your options as <em>safe_value_no_spaces</em>|<em>The Real Value</em>.'));
         return false;
       }
     }
   }

  return true;
}

 /**
 * Generate a string of values from PHP code.  This is used to generate both options and default options.
 * @param $component
 *   An array of information describing the component, directly correlating to
 *   the webform_component database schema.
 * @return
 *   A text string of alternatives.
 */

function webform_php_eval_items($incomming_php){
  global $user;
  
  if (!($user->uid == 1 OR user_access('use PHP for additional processing'))){
    drupal_set_message(t('You do not have permission to use php to generate options'));
    return '';
  }
  // seems like there should be safety checking here, but what can you do if you are going to
  // let the users enter php !!!!
  // drupal_set_message("doing eval: {$incomming_php}");
  $options = eval($incomming_php);
  if (!is_array($options)){
    drupal_set_message("<pre>webform_php_get_items: ERROR the php did not generate an array \n php_options: $incomming_php\n".var_export($options, TRUE).'</pre>');
    return '';
  }
  
  $opt_arr = array();
  foreach ($options as $k => $v) {
      if (is_string($k)){
        $opt_arr[] = "$k|$v";
      }
      else {
        $opt_arr[]= "$v";
      }
  }
  return  $opt_arr;
}


/**
  * Use PHP provided by the user to create the option and default option values. This function also alters
  * the database data in serialised data in webform_component.extra so that data will be interpreted
  * correctly after the form is submitted
 * @param $component
 *   An array of information describing the component, directly correlating to
 *   the webform_component database schema.
 * @return
 *   An array of two text strings which have the new options and default(s)
 */
function webform_php_get_items($component){
  //drupal_set_message("<pre>webform_php_get_items component:\n".print_r($component, TRUE).'</pre>');

  if (strlen($component['extra']['php']['phpdefault'])){
    $default_opt_text = implode(", ",webform_php_eval_items($component['extra']['php']['phpdefault']));
  }
  if (strlen($component['extra']['php']['phpoptions'])){
    $opt_text = implode("\n",webform_php_eval_items($component['extra']['php']['phpoptions']));
  }
  

  //
  // now have the dynamic options and default. We have to stick the new options in the database or they are not
  //     processed correctly
  //
  //probably do not need the database read here since all the info is in $component, but to be safe that
  // there are not additions or deletions in $commponnet, we reread the database to get a pristine copy
  $data_value = db_result(db_query("SELECT DISTINCT extra FROM {webform_component} 
    WHERE nid = %d AND cid = %d", $component['nid'], $component['cid']));
    //          1209763732
  $element_data = unserialize($data_value);
  //drupal_set_message("<pre>unserialized data: \n".print_r($element_data, TRUE).'</pre>');
  $element_data['items'] = $opt_text;
  $re_serialize = serialize($element_data);

  db_query("UPDATE {webform_component} SET extra = '{%s}', value = '{%s}'
    WHERE nid = %d AND cid = %d", $re_serialize, $default_opt_text, $component['nid'], $component['cid'] );  
  

  return array('def' => $default_opt_text, 'opts'=>$opt_text);
}




/**
 * Build a form item array containing all the properties of this component.
 *   if there is PHP code for generating options and defaults stick those in the
 *   live component * @param $component
 *   An array of information describing the component, directly correlating to
 *   the webform_component database schema.
 * @return
 *   An array of a form item to be displayed on the client-side webform.
 */
function _webform_render_select($component) {
  //drupal_set_message( '<pre> webform_render before'.print_r($component, true).'</pre>');
  
  if (strlen($component['extra']['php']['phpoptions']) OR strlen($component['extra']['php']['phpdefault'])){
    $php_results = webform_php_get_items($component);
    $component['value'] = $php_results['def']; // set in the default 
    $component['extra']['items'] = $php_results['opts'];
  }
  //drupal_set_message( '<pre> webform_render after'.print_r($component, true).'</pre>');
  $form_item = array(
    '#title'         => $component['name'],
    '#required'      => $component['mandatory'],
    '#weight'        => $component['weight'],
    '#description'   => _webform_filter_descriptions($component['extra']['description']),
    '#prefix'        => '<div class="webform-component-'. $component['type'] .'" id="webform-component-'. $component['form_key'] .'">',
    '#suffix'        => '</div>',
  );

  // Convert the user-entered options list into an array.
  $default_value = _webform_filter_values($component['value'], NULL, NULL, FALSE);
  $options = _webform_select_options($component['extra']['items']);

  if ($component['extra']['aslist'] == 'Y' && $component['extra']['multiple'] != 'Y') {
    $options = array('' => t('select...')) + $options;
  }

  // Set the component options.
  $form_item['#options'] = $options;

  // Set the default value.
  if ($default_value) {
    // Convert default value to a list if necessary.
    if ($component['extra']['multiple'] == 'Y') {
      $varray = array_filter(explode(',', $default_value));
      foreach ($varray as $key => $v) {
        $form_item['#default_value'][] = $v;
      }
    }
    else {
      $form_item['#default_value'] = $default_value;
    }
  }

  if ($component['extra']['aslist'] == 'Y') {
    // Set display as a select list:
    $form_item['#type'] = 'select';
    if ($component['extra']['multiple'] == 'Y') {
      $form_item['#multiple'] = TRUE;
    }
  }
  else {
    if ($component['extra']['multiple'] == 'Y') {
      // Set display as a checkbox set.
      $form_item['#type'] = 'checkboxes';
      // Drupal 5 hack to correct default values.
      $form_item['#process'] = array('webform_expand_checkboxes' => array());
    }
    else {
      // Set display as a radio set.
      $form_item['#type'] = 'radios';
    }
  }
  return $form_item;
}

/**
 * Drupal 5 hack that properly sets checkbox values in multistep forms. The
 * checkboxes element get the proper values set by FAPI, but individual
 * checkboxes are always set to 0 in a multistep form.
 */
function webform_expand_checkboxes($element) {
  $element = expand_checkboxes($element);
  foreach (element_children($element) as $key) {
    if (empty($element['#value'][$key])) {
      $element[$key]['#value'] = 0;
    }
    else {
      $element[$key]['#value'] = 1;
    }
  }
  return $element;
}

/**
 * Display the result of a textfield submission. The output of this function
 * will be displayed under the "results" tab then "submissions".
 * @param $data
 *   An array of information containing the submission result, directly
 *   correlating to the webform_submitted_data database schema.
 * @param $component
 *   An array of information describing the component, directly correlating to
 *   the webform_component database schema.
 * @return
 *   Textual output formatted for human reading.
 */
function _webform_submission_display_select($data, $component, $enabled = false) {
  $form_item = _webform_render_select($component);
  if ($component['extra']['multiple'] == 'Y') {
    // Set the value as an array.
    foreach ((array)$data['value'] as $key => $value) {
      $form_item['#default_value'][] = $value;
    }
  }
  else {
    // Set the value as a single string.
    foreach ((array)$data['value'] as $value) {
      $form_item['#default_value'] = $value;
    }
  }
  $form_item['#disabled'] = !$enabled;
  return $form_item;
}


/**
 * Convert FAPI 0/1 values into something saveable.
 *
 * @param $data
 *   The POST data associated with the component.
 * @param $component
 *   An array of information describing the component, directly correlating to
 *   the webform_component database schema.
 * @return
 *   Nothing.
 */
function _webform_submit_select(&$data, $component) {
  $options = _webform_select_options($component['extra']['items']);

  if (is_array($data)) {
    foreach ($data as $key => $value) {
      if ($value) {
        $data[$key] = $options[$key];
      }
      else {
        $data[$key] = NULL;
      }
    }
  }
}

/**
 * Format the output of emailed data for this component.
 *
 * @param $data
 *   A string or array of the submitted data.
 * @param $component
 *   An array of information describing the component, directly correlating to
 *   the webform_component database schema.
 * @return
 *   Textual output to be included in the email.
 */
function theme_webform_mail_select($data, $component) {
  // Convert submitted 'safe' values to un-edited, original form.
  $options = _webform_select_options($component['extra']['items']);

  // Generate the output.
  if ($component['extra']['multiple']) {
    $output = $component['name'] .":\n";
    foreach ((array)$data as $value) {
      if ($value) {
        if ($options[$value]) {
          $output .= "    - ". $options[$value] ."\n";
        }
      }
    }
  }
  else {
    if ($options[$data]) {
      $output = $component['name'] .": ". $options[$data] ."\n";
    }
  }
  return $output;
}

/**
 * Module specific instance of hook_help().
 */
function _webform_help_select($section) {
  switch ($section) {
    case 'admin/settings/webform#select_description':
      $output = t("Allows creation of checkboxes, radio buttons, or select menus.");
      break;
  }
  return $output;
}

/**
 * Calculate and returns statistics about results for this component from all
 * submission to this webform. The output of this function will be displayed
 * under the "results" tab then "analysis".
 * @param $component
 *   An array of information describing the component, directly correlating to
 *   the webform_component database schema
 * @return
 *   An array of data rows, each containing a statistic for this component's
 *   submissions.
 */
function _webform_analysis_rows_select($component) {
  $options = _webform_select_options($component['extra']['items']);

  $query = 'SELECT data, count(data) as datacount '.
    ' FROM {webform_submitted_data} '.
    ' WHERE nid = %d '.
    ' AND cid = %d '.
    " AND data != '0' AND data != '' ".
    ' GROUP BY data ';
  $result = db_query($query, $component['nid'], $component['cid']);
  $rows = array();
  while ($data = db_fetch_array($result)) {
    if ($options[$data['data']]) {
      $display_option = $options[$data['data']];
    }
    else {
      $display_option = $data['data'];
    }
    $rows[] = array($display_option, $data['datacount']);
  }
  return $rows;
}

/**
 * Return the result of this component's submission for display in a table. The
 * output of this function will be displayed under the "results" tab then "table".
 * @param $data
 *   An array of information containing the submission result, directly
 *   correlating to the webform_submitted_data database schema
 * @return
 *   Textual output formatted for human reading.
 */
function _webform_table_data_select($data) {
  // Set the value as a single string.
  if (is_array($data['value'])) {
    foreach ($data['value'] as $value) {
      if ($value !== '0') {
        $output .= check_plain($value) ."<br />";
      }
    }
  }
  else {
    $output = check_plain(empty($data['value']['0']) ? "" : $data['value']['0']);
  }
  return $output;
}


/**
 * Return the header information for this component to be displayed in a comma
 * seperated value file. The output of this function will be displayed under the
 * "results" tab then "download".
 * @param $component
 *   An array of information describing the component, directly correlating to
 *   the webform_component database schema.
 * @return
 *   An array of data to be displayed in the first three rows of a CSV file, not
 *   including either prefixed or trailing commas.
 */
function _webform_csv_headers_select($component) {
  $header = array();
  $header[0] = '';

  if ($component['extra']['multiple']) {
    $header[1] = $component['name'];
    $items = _webform_select_options($component['extra']['items']);
    foreach ($items as $key => $item) {
      $header[2] .= "\,". $key;
      // Empty column per sub-field in main header.
      $header[1] .= "\,";
    }
    // Remove the preceding extra comma.
    $header[2] = substr($header[2], 2);
    // Remove the trailing column.
    $header[1] = substr($header[1], 0, -2);
  }
  else {
    $header[2] = $component['name'];
  }
  return $header;
}

/**
 * Return the result of a textfield submission. The output of this function will
 * be displayed under the "results" tab then "submissions".
 * @param $data
 *   An array of information containing the submission result, directly
 *   correlating to the webform_submitted_data database schema.
 * @return
 *   Textual output formatted for CSV, not including either prefixed or trailing
 *   commas.
 */
function _webform_csv_data_select($data, $component) {
  $value = _webform_filter_values($component['value'], NULL, NULL, FALSE);
  $options = _webform_select_options($component['extra']['items']);

  if ($component['extra']['multiple']) {
    foreach ($options as $key => $item) {
      if (in_array($key, (array)$data['value']) === TRUE) {
        $output .= '\,Yes';
      }
      else {
        $output .= '\,No';
      }
    }
    // Remove the preceding extra comma.
    $output = substr($output, 2);
  }
  else {
    $output = $data['value'][0];
  }
  return $output;
}

/**
 * Utility function to split user-entered values from new-line seperated
 * text into an array of options.
 */
function _webform_select_options($text) {
  $options = array();
  $rows = array_filter(explode("\n", _webform_filter_values(trim($text))));
  foreach ($rows as $option) {
    $option = trim($option);
    if (preg_match('/^([^|]+)\|(.*)$/', $option, $matches)) {
      $options[$matches[1]] = $matches[2];
    }
    else {
      $options[$option] = $option;
    }
  }
  return $options;
}
